#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "bhv_mark_move.h"
#include "bhv_defence_move.h"
#include "bhv_basic_tackle.h"

#include <rcsc/action/basic_actions.h>
#include <rcsc/action/body_go_to_point.h>
#include <rcsc/action/body_intercept.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_low_conf_teammate.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>
#include <rcsc/player/intercept_table.h>

#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

#include <rcsc/geom/vector_2d.h>
/*-------------------------------------------------------------------*/
 /*!
  execute action
*/
using namespace rcsc;
bool
Bhv_MarkMove::execute( rcsc::PlayerAgent * agent )
{
    rcsc::dlog.addText( rcsc::Logger::TEAM,
                        "%s:%d: Bhv_MarkMove"
                        ,__FILE__, __LINE__ );

	
	 const rcsc::WorldModel & wm = agent->world();
	const PlayerObject ( const WorldModel & wm,
				    const Vector2D & M_home_pos){
      double dist_opp_to_home = 1000.0;
      const PlayerObject * opp
		=wm.getOpponentNearestTo( M_home_pos, 1,&dist_opp_to_home );

	if( ! opp
	    || (wm.existKickableTeammate()
		&& opp->distFromBall() > 2.5 ) ) {
	printf("mark1!");
	return NULL;
	}
	if ( dist_opp_to_home > 7.0 && M_home_pos.x < opp->pos().x ) {
	printf("mark2!");
	return NULL;
	}
	const PlayerPtrCont::const_iterator 
                 end = wm.teammatesFromSelf().end();
	for ( PlayerPtrCont::const_iterator
	        it = wm.teammatesFromSelf().begin();
	      it != end;
	      ++it ) {
	  if ( (*it)->pos().dist( opp->pos() ) < dist_opp_to_home ){
	       return NULL;
	    }
         }
	return opp;
	}

    AngleDeg block_angle = (wm.ball().pos() - opp->pos() ).th();
    Vector2D block_point
       = opp->pos() + Vector2D::polar2vector( 0.2,block_angle );
    block_point.x -= 0.1;
    if ( block_point.x < wm.self().pos().x - 1.5 ) {
      block_point.y = wm.self().pos().y;
    }

    Body_GoToPoint( block_point,
			1.0,
			ServerParam::i().maxPower(),
			5,
			false,
			true,
			40.0
			).execute( agent );
	

// check ball owner

    int self_min = wm.interceptTable()->selfReachCycle();
    int mate_min = wm.interceptTable()->teammateReachCycle();
    int opp_min = wm.interceptTable()->opponentReachCycle();

    if ( ! wm.existKickableTeammate()
         && (self_min < 3
              ||
               (self_min < mate_min
               && self_min < opp_min +1)
              )
         )
    {
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            "%s:%d: intercept"
                            ,__FILE__, __LINE__ );
        rcsc::Body_Intercept().execute( agent );
        if ( wm.ball().distFromSelf()
             < rcsc::ServerParam::i().visibleDistance() )
        {
            agent->setNeckAction( new rcsc::Neck_TurnToLowConfTeammate() );;
        }
        else
        {
            agent->setNeckAction( new rcsc::Neck_TurnToBallOrScan() );
        }
        return true;
    }




return true;

}


/*-------------------------------------------------------------------*/
/*!

*/
double
Bhv_MarkMove::getDashPower( const rcsc::PlayerAgent * agent,
                             const rcsc::Vector2D & target_point )
{
    static bool s_recover_mode = false;

    const rcsc::WorldModel & wm = agent->world();
    /*--------------------------------------------------------*/
    // check recover

    if ( wm.self().stamina() < rcsc::ServerParam::i().staminaMax() * 0.5 )
    {
        s_recover_mode = true;
    }
    else if ( wm.self().stamina() > rcsc::ServerParam::i().staminaMax() * 0.7 )
    {
        s_recover_mode = false;
    }

    /*--------------------------------------------------------*/
    double dash_power = 100.0;
    const double my_inc
        = wm.self().playerType().staminaIncMax()
        * wm.self().recovery();

    if ( wm.defenseLineX() > wm.self().pos().x
         && wm.ball().pos().x < wm.defenseLineX() + 20.0 )
    {
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            "%s:%d: correct DF line. keep max power"
                            ,__FILE__, __LINE__ );
        // keep max power
        dash_power = rcsc::ServerParam::i().maxPower();
    }
    else if ( s_recover_mode )
    {
        dash_power = my_inc - 25.0; // preffered recover value
        if ( dash_power < 0.0 ) dash_power = 0.0;

        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            "%s:%d: recovering"
                            ,__FILE__, __LINE__ );
    }
    // exist kickable teammate
    else if ( wm.existKickableTeammate()
              && wm.ball().distFromSelf() < 20.0 )
    {
        dash_power = std::min( my_inc * 1.1,
                               rcsc::ServerParam::i().maxPower() );
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            "%s:%d: exist kickable teammate. dash_power= %f"
                            ,__FILE__, __LINE__,
                            dash_power );
    }
    // in offside area
    else if ( wm.self().pos().x > wm.offsideLineX() )
    {
        dash_power = rcsc::ServerParam::i().maxPower();
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            "%s:%d: in offside area. dash_power= %f"
                            ,__FILE__, __LINE__,
                            dash_power );
    }
    // normal
    else
    {
        dash_power = std::min( my_inc * 1.7,
                               rcsc::ServerParam::i().maxPower() );
        rcsc::dlog.addText( rcsc::Logger::TEAM,
                            "%s:%d: normal mode dash_power= %f"
                            ,__FILE__, __LINE__,
                            dash_power );
    }

    return dash_power;
}
